@@ -92,11 +92,11 @@ struct _CcPowerPanelPrivate
   GtkWidget     *automatic_suspend_row;
   GtkWidget     *automatic_suspend_label;
   GtkWidget     *critical_battery_row;
-  GtkWidget     *critical_battery_combo;
 
   GDBusProxy    *bt_rfkill;
   GDBusProxy    *bt_properties;
   GtkWidget     *bt_switch;
+  GtkWidget     *bt_row;
 
 #ifdef HAVE_NETWORK_MANAGER
   NMClient      *nm_client;
@@ -1035,30 +1035,6 @@ combo_time_changed_cb (GtkWidget *widget, CcPowerPanel *self)
   g_settings_set_int (self->priv->gsd_settings, key, value);
 }
 
-static void
-combo_enum_changed_cb (GtkWidget *widget, CcPowerPanel *self)
-{
-  GtkTreeIter iter;
-  GtkTreeModel *model;
-  gint value;
-  gboolean ret;
-  const gchar *key = (const gchar *)g_object_get_data (G_OBJECT(widget), "_gsettings_key");
-
-  /* no selection */
-  ret = gtk_combo_box_get_active_iter (GTK_COMBO_BOX(widget), &iter);
-  if (!ret)
-    return;
-
-  /* get entry */
-  model = gtk_combo_box_get_model (GTK_COMBO_BOX(widget));
-  gtk_tree_model_get (model, &iter,
-                      1, &value,
-                      -1);
-
-  /* set both battery and ac keys */
-  g_settings_set_enum (self->priv->gsd_settings, key, value);
-}
-
 static void
 set_value_for_combo (GtkComboBox *combo_box, gint value)
 {
@@ -1101,7 +1077,7 @@ set_ac_battery_ui_mode (CcPowerPanel *self)
   UpDeviceKind kind;
 
   devices = up_client_get_devices (self->priv->up_client);
-  g_debug ("got %d devices from upower\n", devices->len);
+  g_debug ("got %d devices from upower\n", devices ? devices->len : 0);
 
   for (i = 0; devices != NULL && i < devices->len; i++)
     {
@@ -1167,15 +1143,28 @@ static void
 bt_powered_state_changed (CcPowerPanel *panel)
 {
   CcPowerPanelPrivate *priv = panel->priv;
-  gboolean powered;
+  gboolean powered, has_airplane_mode;
   GVariant *v;
 
+  v = g_dbus_proxy_get_cached_property (priv->bt_rfkill, "BluetoothHasAirplaneMode");
+  has_airplane_mode = g_variant_get_boolean (v);
+  g_variant_unref (v);
+
+  if (!has_airplane_mode)
+    {
+      g_debug ("BluetoothHasAirplaneMode is false, hiding Bluetooth power row");
+      gtk_widget_hide (priv->bt_row);
+      return;
+    }
+
   v = g_dbus_proxy_get_cached_property (priv->bt_rfkill, "BluetoothAirplaneMode");
   powered = !g_variant_get_boolean (v);
   g_variant_unref (v);
 
   g_debug ("bt powered state changed to %s", powered ? "on" : "off");
 
+  gtk_widget_show (priv->bt_row);
+
   g_signal_handlers_block_by_func (priv->bt_switch, bt_switch_changed, panel);
   gtk_switch_set_active (GTK_SWITCH (priv->bt_switch), powered);
   g_signal_handlers_unblock_by_func (priv->bt_switch, bt_switch_changed, panel);
@@ -1655,33 +1644,37 @@ add_power_saving_section (CcPowerPanel *self)
 							   "/org/gnome/SettingsDaemon/Rfkill",
 							   "org.freedesktop.DBus.Properties",
 							   NULL, NULL);
-      row = no_prelight_row_new ();
-      box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 50);
-      gtk_container_add (GTK_CONTAINER (row), box);
-      label = gtk_label_new (_("_Bluetooth"));
-      gtk_widget_set_halign (label, GTK_ALIGN_START);
-      gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
-      gtk_widget_set_margin_start (label, 20);
-      gtk_widget_set_margin_end (label, 20);
-      gtk_widget_set_margin_top (label, 6);
-      gtk_widget_set_margin_bottom (label, 6);
-      gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
-
-      priv->bt_switch = sw = gtk_switch_new ();
-      gtk_widget_set_margin_start (sw, 20);
-      gtk_widget_set_margin_end (sw, 20);
-      gtk_widget_set_valign (sw, GTK_ALIGN_CENTER);
-      gtk_box_pack_start (GTK_BOX (box), sw, FALSE, TRUE, 0);
-      gtk_label_set_mnemonic_widget (GTK_LABEL (label), sw);
-      gtk_container_add (GTK_CONTAINER (widget), row);
-      gtk_size_group_add_widget (priv->row_sizegroup, row);
-      g_signal_connect_swapped (G_OBJECT (priv->bt_rfkill), "g-properties-changed",
-				G_CALLBACK (bt_powered_state_changed), self);
-      g_signal_connect (G_OBJECT (priv->bt_switch), "notify::active",
-			G_CALLBACK (bt_switch_changed), self);
-
-      bt_powered_state_changed (self);
     }
+
+  row = no_prelight_row_new ();
+  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 50);
+  gtk_container_add (GTK_CONTAINER (row), box);
+  label = gtk_label_new (_("_Bluetooth"));
+  gtk_widget_set_halign (label, GTK_ALIGN_START);
+  gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
+  gtk_widget_set_margin_start (label, 20);
+  gtk_widget_set_margin_end (label, 20);
+  gtk_widget_set_margin_top (label, 6);
+  gtk_widget_set_margin_bottom (label, 6);
+  gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
+
+  priv->bt_switch = sw = gtk_switch_new ();
+  gtk_widget_set_margin_start (sw, 20);
+  gtk_widget_set_margin_end (sw, 20);
+  gtk_widget_set_valign (sw, GTK_ALIGN_CENTER);
+  gtk_box_pack_start (GTK_BOX (box), sw, FALSE, TRUE, 0);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), sw);
+  gtk_container_add (GTK_CONTAINER (widget), row);
+  gtk_size_group_add_widget (priv->row_sizegroup, row);
+  gtk_widget_show_all (box);
+  gtk_widget_set_no_show_all (row, TRUE);
+  priv->bt_row = row;
+  g_signal_connect_swapped (G_OBJECT (priv->bt_rfkill), "g-properties-changed",
+			G_CALLBACK (bt_powered_state_changed), self);
+  g_signal_connect (G_OBJECT (priv->bt_switch), "notify::active",
+		G_CALLBACK (bt_switch_changed), self);
+
+  bt_powered_state_changed (self);
 #endif
 
   gtk_widget_show_all (widget);
@@ -1734,15 +1727,6 @@ on_suspend_settings_changed (GSettings    *settings,
                              const char   *key,
                              CcPowerPanel *self)
 {
-  CcPowerPanelPrivate *priv = self->priv;
-  gint value;
-
-  if (g_strcmp0 (key, "critical-battery-action") == 0 &&
-      priv->critical_battery_combo != NULL)
-    {
-      value = g_settings_get_enum (settings, "critical-battery-action");
-      set_value_for_combo (GTK_COMBO_BOX (priv->critical_battery_combo), value);
-    }
   if (g_str_has_prefix (key, "sleep-inactive-"))
     {
       update_automatic_suspend_label (self);
@@ -1817,12 +1801,11 @@ add_automatic_suspend_section (CcPowerPanel *self)
   GtkWidget *sw, *row;
   gchar *s;
   gint value;
-  GtkTreeModel *model;
   GtkWidget *dialog;
   GtkWidget *combo;
-  GtkCellRenderer *cell;
   GVariant *result;
   GDBusConnection *connection;
+  const char *critical_battery_str;
 
   /* The default values for these settings are unfortunate for us;
    * timeout == 0, action == suspend means 'do nothing' - just
@@ -1885,6 +1868,7 @@ add_automatic_suspend_section (CcPowerPanel *self)
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 50);
   gtk_container_add (GTK_CONTAINER (row), box);
   label = gtk_label_new (_("_Automatic suspend"));
+  atk_object_set_name (ATK_OBJECT (gtk_widget_get_accessible (self->priv->automatic_suspend_row)), _("Automatic suspend"));
   gtk_widget_set_halign (label, GTK_ALIGN_START);
   gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
   gtk_widget_set_margin_start (label, 20);
@@ -1917,14 +1901,12 @@ add_automatic_suspend_section (CcPowerPanel *self)
   gtk_widget_set_margin_bottom (label, 6);
   gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
 
-  value = 0;
-
   connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, NULL);
   result = g_dbus_connection_call_sync (connection,
-                                        "org.freedesktop.login1",
-                                        "/org/freedesktop/login1",
-                                        "org.freedesktop.login1.Manager",
-                                        "CanHibernate",
+                                        "org.freedesktop.UPower",
+                                        "/org/freedesktop/UPower",
+                                        "org.freedesktop.UPower",
+                                        "GetCriticalAction",
                                         NULL,
                                         NULL,
                                         G_DBUS_CALL_FLAGS_NONE,
@@ -1935,44 +1917,39 @@ add_automatic_suspend_section (CcPowerPanel *self)
 
   if (result)
     {
-      GVariant *result_variant = g_variant_get_child_value (result, 0);
-      if (g_strcmp0 (g_variant_get_string (result_variant, NULL), "yes") == 0)
-        value = 1;
+      GVariant *result_variant;
+      const char *str;
+
+      result_variant = g_variant_get_child_value (result, 0);
+      str = g_variant_get_string (result_variant, NULL);
+
+      if (g_strcmp0 (str, "Hibernate") == 0 ||
+          g_strcmp0 (str, "HybridSleep") == 0)
+        critical_battery_str = _("Hibernate");
+      else if (g_strcmp0 (str, "PowerOff") == 0)
+        critical_battery_str = _("Power Off");
+      else
+        {
+          g_debug ("Unexpected value returned by UPower for GetCriticalAction(): %s", str);
+          critical_battery_str = _("Power Off");
+        }
 
       g_variant_unref(result_variant);
       g_variant_unref(result);
     }
-
-  if (value)
-    {
-      model = (GtkTreeModel*)gtk_builder_get_object (priv->builder, "liststore_critical");
-      priv->critical_battery_combo = sw = gtk_combo_box_new_with_model (model);
-      gtk_label_set_mnemonic_widget (GTK_LABEL (label), sw);
-      cell = gtk_cell_renderer_text_new ();
-      gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (sw), cell, TRUE);
-      gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (sw), cell, "text", 0);
-      gtk_widget_set_margin_start (sw, 20);
-      gtk_widget_set_margin_end (sw, 20);
-      gtk_widget_set_valign (sw, GTK_ALIGN_CENTER);
-
-      g_object_set_data (G_OBJECT (sw), "_gsettings_key", "critical-battery-action");
-      value = g_settings_get_enum (priv->gsd_settings, "critical-battery-action");
-      set_value_for_combo (GTK_COMBO_BOX (sw), value);
-      g_signal_connect (sw, "changed",
-                        G_CALLBACK (combo_enum_changed_cb), self);
-
-      gtk_box_pack_start (GTK_BOX (box), sw, FALSE, TRUE, 0);
-    }
   else
     {
-      label = gtk_label_new (_("Power Off"));
-      gtk_widget_set_margin_start (label, 20);
-      gtk_widget_set_margin_end (label, 20);
-      gtk_widget_set_margin_top (label, 6);
-      gtk_widget_set_margin_bottom (label, 6);
-      gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
+      g_debug ("GetCriticalAction() call failed, is UPower correctly installed?");
+      critical_battery_str = _("Power Off");
     }
 
+  label = gtk_label_new (critical_battery_str);
+  gtk_widget_set_margin_start (label, 20);
+  gtk_widget_set_margin_end (label, 20);
+  gtk_widget_set_margin_top (label, 6);
+  gtk_widget_set_margin_bottom (label, 6);
+  gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
+
   gtk_container_add (GTK_CONTAINER (widget), row);
   gtk_size_group_add_widget (priv->row_sizegroup, row);
   gtk_widget_show_all (widget);
